{
  Copyright (C) 2015 by Clever Components

  Author: Sergey Shirokov <admin@clevercomponents.com>

  Website: www.CleverComponents.com

  This file is part of Json Serializer.

  Json Serializer is free software: you can redistribute it and/or modify
  it under the terms of the GNU Lesser General Public License version 3
  as published by the Free Software Foundation and appearing in the
  included file COPYING.LESSER.

  Json Serializer is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public License
  along with Json Serializer. If not, see <http://www.gnu.org/licenses/>.

  The current version of Json Serializer needs for the non-free library
  Clever Internet Suite. This is a drawback, and we suggest the task of changing
  the program so that it does the same job without the non-free library.
  Anyone who thinks of doing substantial further work on the program,
  first may free it from dependence on the non-free library.
}

unit clJsonSerializerTests;

interface

uses
  System.Classes, System.Generics.Collections, System.SysUtils, TestFramework, clJsonSerializer;

type
  TclTestUnsupportedType = class
  strict private
    FFloatValue: Double;
  public
    [TclJsonProperty('floatValue')]
    property FloatValue: Double read FFloatValue write FFloatValue;
  end;

  TclTestUnsupportedArrayType = class
  strict private
    FFloatArray: TArray<Double>;
  public
    [TclJsonProperty('floatArray')]
    property FloatArray: TArray<Double> read FFloatArray write FFloatArray;
  end;

  TclTestSubObject = class
  strict private
    FName: string;
    FValue: string;
  public
    [TclJsonString('name')]
    property Name: string read FName write FName;
    [TclJsonString('value')]
    property Value: string read FValue write FValue;
  end;

  TclTestObject = class
  strict private
    FBooleanValue: Boolean;
    FNonSerializable: string;
    FStringValue: string;
    FValue: string;
    FIntegerValue: Integer;
    FSubObject: TclTestSubObject;
    FIntArray: TArray<Integer>;
    FStrArray: TArray<string>;
    FBoolArray: TArray<Boolean>;
    FObjArray: TArray<TclTestSubObject>;

    procedure SetSubObject(const Value: TclTestSubObject);
    procedure SetObjArray(const Value: TArray<TclTestSubObject>);
  public
    constructor Create;
    destructor Destroy; override;

    [TclJsonString('stringValue')]
    property StringValue: string read FStringValue write FStringValue;

    [TclJsonProperty('integerValue')]
    property IntegerValue: Integer read FIntegerValue write FIntegerValue;

    [TclJsonProperty('value')]
    property Value: string read FValue write FValue;

    [TclJsonProperty('booleanValue')]
    property BooleanValue: Boolean read FBooleanValue write FBooleanValue;

    [TclJsonProperty('subObject')]
    property SubObject: TclTestSubObject read FSubObject write SetSubObject;

    [TclJsonProperty('intArray')]
    property IntArray: TArray<Integer> read FIntArray write FIntArray;

    [TclJsonString('strArray')]
    property StrArray: TArray<string> read FStrArray write FStrArray;

    [TclJsonProperty('boolArray')]
    property BoolArray: TArray<Boolean> read FBoolArray write FBoolArray;

    [TclJsonProperty('objArray')]
    property ObjArray: TArray<TclTestSubObject> read FObjArray write SetObjArray;

    property NonSerializable: string read FNonSerializable write FNonSerializable;
  end;

  TclJsonSerializerTests = class(TTestCase)
  published
    procedure TestDeserialize;
    procedure TestSerialize;
    procedure TestUnsupportedType;
  end;


implementation

{ TclJsonSerializerTests }

procedure TclJsonSerializerTests.TestDeserialize;
const
  jsonEtalon = '{"stringValue": "qwe", "integerValue": 123, "value": asd, "booleanValue": true}';
  jsonEtalon2 = '{"stringValue": "qwe", "subObject": {"name": "qwerty"}, "intArray": [111, 222], "strArray": ["val 1", "val 2"], ' +
'"boolArray": [true, false], "objArray": [{"name": "an1"}, {"name": "an2"}]}';

var
  obj: TclTestObject;
begin
  obj := nil;
  try
    obj := TclJsonSerializer.JsonToObject(TclTestObject, jsonEtalon) as TclTestObject;

    CheckEquals('qwe', obj.StringValue);
    CheckEquals(123, obj.IntegerValue);
    CheckEquals('asd', obj.Value);
    CheckEquals(True, obj.BooleanValue);

    FreeAndNil(obj);

    obj := TclJsonSerializer.JsonToObject(TclTestObject, jsonEtalon2) as TclTestObject;

    CheckEquals('qwe', obj.StringValue);

    CheckTrue(obj.SubObject <> nil);
    CheckEquals('qwerty', obj.SubObject.Name);

    CheckEquals(2, Length(obj.IntArray));
    CheckEquals(111, obj.IntArray[0]);
    CheckEquals(222, obj.IntArray[1]);

    CheckEquals(2, Length(obj.StrArray));
    CheckEquals('val 1', obj.StrArray[0]);
    CheckEquals('val 2', obj.StrArray[1]);

    CheckEquals(2, Length(obj.BoolArray));
    CheckEquals(True, obj.BoolArray[0]);
    CheckEquals(False, obj.BoolArray[1]);

    CheckEquals(2, Length(obj.ObjArray));

    CheckTrue(obj.ObjArray[0] <> nil);
    CheckEquals('an1', obj.ObjArray[0].Name);

    CheckTrue(obj.ObjArray[1] <> nil);
    CheckEquals('an2', obj.ObjArray[1].Name);
  finally
    obj.Free();
  end;
end;

procedure TclJsonSerializerTests.TestSerialize;
const
  jsonEtalon = '{"stringValue": "qwe", "integerValue": 123, "value": asd, "booleanValue": true}';
  jsonEtalon2 = '{"stringValue": "qwe", "integerValue": 123, "value": asd, "booleanValue": true, ' +
'"subObject": {"name": "qwerty"}, "intArray": [111, 222], "strArray": ["val 1", "val 2"], ' +
'"boolArray": [true, false], "objArray": [{"name": "an1"}, {"name": "an2"}]}';

var
  obj: TclTestObject;
  json: string;
  intArr: TArray<Integer>;
  strArr: TArray<string>;
  boolArr: TArray<Boolean>;
  objArr: TArray<TclTestSubObject>;
begin
  obj := TclTestObject.Create();
  try
    obj.StringValue := 'qwe';
    obj.IntegerValue := 123;
    obj.Value := 'asd';
    obj.BooleanValue := True;
    obj.NonSerializable := 'zxc';

    json := TclJsonSerializer.ObjectToJson(obj);

    CheckEquals(jsonEtalon, json);

    obj.SubObject := TclTestSubObject.Create();
    obj.SubObject.Name := 'qwerty';

    SetLength(intArr, 2);
    obj.IntArray := intArr;
    intArr[0] := 111;
    intArr[1] := 222;

    SetLength(strArr, 2);
    obj.StrArray := strArr;
    strArr[0] := 'val 1';
    strArr[1] := 'val 2';

    SetLength(boolArr, 2);
    obj.BoolArray := boolArr;
    boolArr[0] := True;
    boolArr[1] := False;

    SetLength(objArr, 2);
    obj.ObjArray := objArr;
    objArr[0] := TclTestSubObject.Create();
    objArr[0].Name := 'an1';
    objArr[1] := TclTestSubObject.Create();
    objArr[1].Name := 'an2';

    json := TclJsonSerializer.ObjectToJson(obj);

    CheckEquals(jsonEtalon2, json);
  finally
    obj.Free();
  end;
end;

procedure TclJsonSerializerTests.TestUnsupportedType;
var
  obj: TclTestUnsupportedType;
  objArr: TclTestUnsupportedArrayType;
  arr: TArray<Double>;
begin
  obj := nil;
  objArr := nil;
  try
    obj := TclTestUnsupportedType.Create();
    obj.FloatValue := 12.5;

    try
      TclJsonSerializer.ObjectToJson(obj);
      Fail('Data type checking does not work');
    except
      on EclJsonSerializerError do;
    end;
    FreeAndNil(obj);

    try
      obj := TclJsonSerializer.JsonToObject(TclTestUnsupportedType, '{"floatValue": 12}') as TclTestUnsupportedType;
      Fail('Data type checking does not work');
    except
      on EclJsonSerializerError do;
    end;
    FreeAndNil(obj);

    objArr := TclTestUnsupportedArrayType.Create();
    SetLength(arr, 1);
    objArr.FloatArray := arr;
    objArr.FloatArray[0] := 12.5;

    try
      TclJsonSerializer.ObjectToJson(objArr);
      Fail('Data type checking does not work');
    except
      on EclJsonSerializerError do;
    end;
    FreeAndNil(objArr);

    try
      objArr := TclJsonSerializer.JsonToObject(TclTestUnsupportedArrayType, '{"floatArray": [11, 22]}') as TclTestUnsupportedArrayType;
      Fail('Data type checking does not work');
    except
      on EclJsonSerializerError do;
    end;
  finally
    objArr.Free();
    obj.Free();
  end;
end;

{ TclTestObject }

constructor TclTestObject.Create;
begin
  inherited Create();

  FSubObject := nil;
  FIntArray := nil;
  FStrArray := nil;
  FBoolArray := nil;
  FObjArray := nil;
end;

destructor TclTestObject.Destroy;
begin
  SetObjArray(nil);

  FSubObject.Free();

  inherited Destroy();
end;

procedure TclTestObject.SetObjArray(const Value: TArray<TclTestSubObject>);
var
  obj: TObject;
begin
  if (FObjArray <> nil) then
  begin
    for obj in FObjArray do
    begin
      obj.Free();
    end;
  end;

  FObjArray := Value;
end;

procedure TclTestObject.SetSubObject(const Value: TclTestSubObject);
begin
  FSubObject.Free();
  FSubObject := Value;
end;

initialization
  TestFramework.RegisterTest(TclJsonSerializerTests.Suite);

end.
